@@ -2,7 +2,6 @@
 
 import json
 import os
-import sys
 
 import arrow
 from click import get_app_dir
@@ -12,10 +11,10 @@ import requests
 
 from watson import Watson, WatsonError
 from watson.watson import ConfigParser, ConfigurationError
+from watson.utils import PY2
 
 from . import mock_read
 
-PY2 = sys.version_info[0] == 2
 TEST_FIXTURE_DIR = py.path.local(
     os.path.dirname(
         os.path.realpath(__file__)
@@ -28,6 +27,13 @@ else:
     builtins = '__builtin__'
 
 
+@pytest.fixture
+def json_mock(mock):
+    return mock.patch.object(
+        json, 'dumps', side_effect=json.dumps, autospec=True
+    )
+
+
 # NOTE: All timestamps need to be > 3600 to avoid breaking the tests on
 # Windows.
 
@@ -265,6 +271,15 @@ my project = A B
     assert watson.current['tags'] == ['C', 'D', 'A', 'B']
 
 
+def test_start_nogap(mock, watson):
+
+    watson.start('foo')
+    watson.stop()
+    watson.start('bar', gap=False)
+
+    assert watson.frames[-1].stop == watson.current['start']
+
+
 # stop
 
 def test_stop_started_project(watson):
@@ -298,6 +313,24 @@ def test_stop_no_project(watson):
         watson.stop()
 
 
+def test_stop_started_project_at(watson):
+    watson.start('foo')
+    now = arrow.now()
+
+    with pytest.raises(ValueError):
+        time_str = '1970-01-01T00:00'
+        time_obj = arrow.get(time_str)
+        watson.stop(stop_at=time_obj)
+
+    with pytest.raises(ValueError):
+        time_str = '2999-31-12T23:59'
+        time_obj = arrow.get(time_str)
+        watson.stop(stop_at=time_obj)
+
+    watson.stop(stop_at=now)
+    assert watson.frames[-1].stop == now
+
+
 # cancel
 
 def test_cancel_started_project(watson):
@@ -315,19 +348,17 @@ def test_cancel_no_project(watson):
 
 # save
 
-def test_save_without_changes(mock, watson):
+def test_save_without_changes(mock, watson, json_mock):
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert not json_mock.called
 
 
-def test_save_current(mock, watson):
+def test_save_current(mock, watson, json_mock):
     watson.start('foo', ['A', 'B'])
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert json_mock.call_count == 1
@@ -337,11 +368,10 @@ def test_save_current(mock, watson):
     assert result['tags'] == ['A', 'B']
 
 
-def test_save_current_without_tags(mock, watson):
+def test_save_current_without_tags(mock, watson, json_mock):
     watson.start('foo')
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert json_mock.call_count == 1
@@ -354,37 +384,41 @@ def test_save_current_without_tags(mock, watson):
     assert dump_args['ensure_ascii'] is False
 
 
-def test_save_empty_current(config_dir, mock):
-    watson = Watson(current={'project': 'foo', 'start': 4000},
-                    config_dir=config_dir)
-    watson.current = {}
+def test_save_empty_current(config_dir, mock, json_mock):
+    watson = Watson(current={}, config_dir=config_dir)
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
+
+    watson.current = {'project': 'foo', 'start': 4000}
     watson.save()
 
     assert json_mock.call_count == 1
     result = json_mock.call_args[0][0]
+    assert result == {'project': 'foo', 'start': 4000, 'tags': []}
+
+    watson.current = {}
+    watson.save()
+
+    assert json_mock.call_count == 2
+    result = json_mock.call_args[0][0]
     assert result == {}
 
 
-def test_save_frames_no_change(config_dir, mock):
+def test_save_frames_no_change(config_dir, mock, json_mock):
     watson = Watson(frames=[[4000, 4010, 'foo', None]],
                     config_dir=config_dir)
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert not json_mock.called
 
 
-def test_save_added_frame(config_dir, mock):
+def test_save_added_frame(config_dir, mock, json_mock):
     watson = Watson(frames=[[4000, 4010, 'foo', None]], config_dir=config_dir)
     watson.frames.add('bar', 4010, 4020, ['A'])
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert json_mock.call_count == 1
@@ -396,13 +430,12 @@ def test_save_added_frame(config_dir, mock):
     assert result[1][4] == ['A']
 
 
-def test_save_changed_frame(config_dir, mock):
+def test_save_changed_frame(config_dir, mock, json_mock):
     watson = Watson(frames=[[4000, 4010, 'foo', None, ['A']]],
                     config_dir=config_dir)
     watson.frames[0] = ('bar', 4000, 4010, ['A', 'B'])
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert json_mock.call_count == 1
@@ -432,24 +465,22 @@ def test_save_config(mock, watson):
     assert write_mock.call_count == 1
 
 
-def test_save_last_sync(mock, watson):
+def test_save_last_sync(mock, watson, json_mock):
     now = arrow.now()
     watson.last_sync = now
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert json_mock.call_count == 1
     assert json_mock.call_args[0][0] == now.timestamp
 
 
-def test_save_empty_last_sync(config_dir, mock):
+def test_save_empty_last_sync(config_dir, mock, json_mock):
     watson = Watson(last_sync=arrow.now(), config_dir=config_dir)
     watson.last_sync = None
 
     mock.patch('%s.open' % builtins, mock.mock_open())
-    json_mock = mock.patch('json.dump')
     watson.save()
 
     assert json_mock.call_count == 1
@@ -808,3 +839,27 @@ def test_rename_tag_with_time(mock, watson):
     assert watson.frames[1].tags == ['baz']
     # assert watson.frames[1].updated_at.timestamp == 9000
     assert watson.frames[1].updated_at.timestamp > 4035
+
+# add
+
+
+def test_add_success(mock, watson):
+    """
+    Adding a new frame outside of live tracking successfully
+    """
+    watson.add(project="test_project", tags=['fuu', 'bar'],
+               from_date=6000, to_date=7000)
+
+    assert len(watson.frames) == 1
+    assert watson.frames[0].project == "test_project"
+    assert 'fuu' in watson.frames[0].tags
+
+
+def test_add_failure(mock, watson):
+    """
+    Adding a new frame outside of live tracking fails when
+    to date is before from date
+    """
+    with pytest.raises(WatsonError):
+        watson.add(project="test_project", tags=['fuu', 'bar'],
+                   from_date=7000, to_date=6000)
