@@ -5,6 +5,7 @@ import itertools
 import json
 import operator
 import os
+import re
 
 from dateutil import tz
 from functools import reduce
@@ -14,9 +15,9 @@ import click
 
 from . import watson as _watson
 from .frames import Frame
-from .utils import (format_timedelta, get_frame_from_argument,
-                    get_start_time_for_period, options, safe_save,
-                    sorted_groupby, style)
+from .utils import (apply_weekday_offset, format_timedelta,
+                    get_frame_from_argument, get_start_time_for_period,
+                    options, safe_save, sorted_groupby, style, parse_tags)
 
 
 class MutuallyExclusiveOption(click.Option):
@@ -57,10 +58,44 @@ class DateParamType(click.ParamType):
             # expected by the user, so that midnight is midnight in the local
             # timezone, not in UTC. Cf issue #16.
             date.tzinfo = tz.tzlocal()
+
+            # Add an offset to match the week beginning specified in the
+            # configuration
+            if param.name == "week":
+                week_start = ctx.obj.config.get(
+                    "options", "week_start", "monday")
+                date = apply_weekday_offset(
+                    start_time=date, week_start=week_start)
             return date
 
 
+class TimeParamType(click.ParamType):
+    name = 'time'
+
+    def convert(self, value, param, ctx):
+        if isinstance(value, arrow.Arrow):
+            return value
+
+        date_pattern = r'\d{4}-\d\d-\d\d'
+        time_pattern = r'\d\d:\d\d(:\d\d)?'
+
+        if re.match('^{time_pat}$'.format(time_pat=time_pattern), value):
+            cur_date = arrow.now().date().isoformat()
+            cur_time = '{date}T{time}'.format(date=cur_date, time=value)
+        elif re.match('^{date_pat}T{time_pat}'.format(
+                date_pat=date_pattern, time_pat=time_pattern), value):
+            cur_time = value
+        else:
+            errmsg = ('Could not parse time.'
+                      'Please specify in (YYYY-MM-DDT)?HH:MM(:SS)? format.')
+            raise WatsonCliError(errmsg)
+
+        local_tz = tz.tzlocal()
+        return arrow.get(cur_time).replace(tzinfo=local_tz)
+
+
 Date = DateParamType()
+Time = TimeParamType()
 
 
 @click.group()
@@ -94,17 +129,17 @@ def help(ctx, command):
     cmd = cli.get_command(ctx, command)
 
     if not cmd:
-        raise click.ClickException("No such command: {}".format(command))
+        raise click.ClickException(u"No such command: {}".format(command))
 
     click.echo(cmd.get_help(ctx))
 
 
-def _start(watson, project, tags, restart=False):
+def _start(watson, project, tags, restart=False, gap=True):
     """
     Start project with given list of tags and save status.
     """
-    current = watson.start(project, tags, restart=restart)
-    click.echo("Starting project {}{} at {}".format(
+    current = watson.start(project, tags, restart=restart, gap=gap)
+    click.echo(u"Starting project {}{} at {}".format(
         style('project', project),
         (" " if current['tags'] else "") + style('tags', current['tags']),
         style('time', "{:HH:mm}".format(current['start']))
@@ -113,10 +148,13 @@ def _start(watson, project, tags, restart=False):
 
 
 @cli.command()
+@click.option('-g/-G', '--gap/--no-gap', 'gap_', is_flag=True, default=True,
+              help=("(Don't) leave gap between end time of previous project "
+                    "and start time of the current."))
 @click.argument('args', nargs=-1)
 @click.pass_obj
 @click.pass_context
-def start(ctx, watson, args):
+def start(ctx, watson, args, gap_=True):
     """
     Start monitoring time for the given project.
     You can add tags indicating more specifically what you are working on with
@@ -126,52 +164,62 @@ def start(ctx, watson, args):
     `options.stop_on_start` is set to a true value (`1`, `on`, `true` or
     `yes`), it is stopped before the new project is started.
 
+    If the '--no-gap' flag is given, the start time of the new project is set
+    to the stop time of the most recently stopped project.
+
     Example:
 
     \b
-    $ watson start apollo11 +module +brakes
+    $ watson start apollo11 +module +brakes --no-gap
     Starting project apollo11 [module, brakes] at 16:34
     """
     project = ' '.join(
         itertools.takewhile(lambda s: not s.startswith('+'), args)
     )
 
-    # Find all the tags starting by a '+', even if there are spaces in them,
-    # then strip each tag and filter out the empty ones
-    tags = list(filter(None, map(operator.methodcaller('strip'), (
-        # We concatenate the word with the '+' to the following words
-        # not starting with a '+'
-        w[1:] + ' ' + ' '.join(itertools.takewhile(
-            lambda s: not s.startswith('+'), args[i + 1:]
-        ))
-        for i, w in enumerate(args) if w.startswith('+')
-    ))))  # pile of pancakes !
+    # Parse all the tags
+    tags = parse_tags(args)
+
+    if project and watson.is_started and not gap_:
+        current = watson.current
+        errmsg = ("Project {} is already started and '--no-gap' is passed. "
+                  "Please stop manually.")
+        raise _watson.WatsonError(errmsg.format(current['project']))
 
     if (project and watson.is_started and
             watson.config.getboolean('options', 'stop_on_start')):
         ctx.invoke(stop)
 
-    _start(watson, project, tags)
+    _start(watson, project, tags, gap=gap_)
 
 
-@cli.command()
+@cli.command(context_settings={'ignore_unknown_options': True})
+@click.option('--at', 'at_', type=Time, default=None,
+              help=('Stop frame at this time. Must be in '
+                    '(YYYY-MM-DDT)?HH:MM(:SS)? format.'))
 @click.pass_obj
-def stop(watson):
+def stop(watson, at_):
     """
     Stop monitoring time for the current project.
 
+    If '--at' option is given, the provided stopping time is used. The
+    specified time must be after the begin of the to be ended frame and must
+    not be in the future.
+
     Example:
 
     \b
-    $ watson stop
-    Stopping project apollo11, started a minute ago. (id: e7ccd52)
+    $ watson stop --at 13:37
+    Stopping project apollo11, started an hour ago and stopped 30 minutes ago. (id: e9ccd52) # noqa: E501
     """
-    frame = watson.stop()
-    click.echo("Stopping project {}{}, started {}. (id: {})".format(
+    frame = watson.stop(stop_at=at_)
+    output_str = u"Stopping project {}{}, started {} and stopped {}. (id: {})"
+    click.echo(output_str.format(
         style('project', frame.project),
         (" " if frame.tags else "") + style('tags', frame.tags),
         style('time', frame.start.humanize()),
-        style('short_id', frame.id)
+        style('time', frame.stop.humanize()),
+        style('short_id', frame.id),
     ))
     watson.save()
 
@@ -223,7 +271,7 @@ def restart(ctx, watson, frame, stop_):
         else:
             # Raise error here, instead of in watson.start(), otherwise
             # will give misleading error if running frame is the first one
-            raise click.ClickException("{} {} {}".format(
+            raise click.ClickException(u"{} {} {}".format(
                 style('error', "Project already started:"),
                 style('project', watson.current['project']),
                 style('tags', watson.current['tags'])))
@@ -241,7 +289,7 @@ def cancel(watson):
     not be recorded.
     """
     old = watson.cancel()
-    click.echo("Canceling the timer for project {}{}".format(
+    click.echo(u"Canceling the timer for project {}{}".format(
         style('project', old['project']),
         (" " if old['tags'] else "") + style('tags', old['tags'])
     ))
@@ -277,32 +325,32 @@ def status(watson, project, tags, elapsed):
     Project apollo11 [brakes] started a minute ago (19.05.2014 at 02:32 PM)
     """
     if not watson.is_started:
-        click.echo("No project started")
+        click.echo("No project started.")
         return
 
     current = watson.current
 
     if project:
-        click.echo("{}".format(
+        click.echo(u"{}".format(
             style('project', current['project']),
         ))
         return
 
     if tags:
-        click.echo("{}".format(
+        click.echo(u"{}".format(
             style('tags', current['tags'])
         ))
         return
 
     if elapsed:
-        click.echo("{}".format(
+        click.echo(u"{}".format(
             style('time', current['start'].humanize())
         ))
         return
 
     datefmt = watson.config.get('options', 'date_format', '%Y.%m.%d')
     timefmt = watson.config.get('options', 'time_format', '%H:%M:%S%z')
-    click.echo("Project {}{} started {} ({} {})".format(
+    click.echo(u"Project {}{} started {} ({} {})".format(
         style('project', current['project']),
         (" " if current['tags'] else "") + style('tags', current['tags']),
         style('time', current['start'].humanize()),
@@ -311,7 +359,10 @@ def status(watson, project, tags, elapsed):
     ))
 
 
-_SHORTCUT_OPTIONS = ['all', 'year', 'month', 'week', 'day']
+_SHORTCUT_OPTIONS = ['all', 'year', 'month', 'luna', 'week', 'day']
+_SHORTCUT_OPTIONS_VALUES = {
+    k: get_start_time_for_period(k) for k in _SHORTCUT_OPTIONS
+}
 
 
 @cli.command()
@@ -328,24 +379,28 @@ _SHORTCUT_OPTIONS = ['all', 'year', 'month', 'week', 'day']
               help="The date at which the report should stop (inclusive). "
               "Defaults to tomorrow.")
 @click.option('-y', '--year', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('year'),
-              mutually_exclusive=['day', 'week', 'month', 'all'],
+              flag_value=_SHORTCUT_OPTIONS_VALUES['year'],
+              mutually_exclusive=['day', 'week', 'luna', 'month', 'all'],
               help='Reports activity for the current year.')
 @click.option('-m', '--month', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('month'),
-              mutually_exclusive=['day', 'week', 'year', 'all'],
+              flag_value=_SHORTCUT_OPTIONS_VALUES['month'],
+              mutually_exclusive=['day', 'week', 'luna', 'year', 'all'],
               help='Reports activity for the current month.')
+@click.option('-l', '--luna', cls=MutuallyExclusiveOption, type=Date,
+              flag_value=_SHORTCUT_OPTIONS_VALUES['luna'],
+              mutually_exclusive=['day', 'week', 'month', 'year', 'all'],
+              help='Reports activity for the current moon cycle.')
 @click.option('-w', '--week', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('week'),
-              mutually_exclusive=['day', 'month', 'year', 'all'],
+              flag_value=_SHORTCUT_OPTIONS_VALUES['week'],
+              mutually_exclusive=['day', 'month', 'luna', 'year', 'all'],
               help='Reports activity for the current week.')
 @click.option('-d', '--day', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('day'),
-              mutually_exclusive=['week', 'month', 'year', 'all'],
+              flag_value=_SHORTCUT_OPTIONS_VALUES['day'],
+              mutually_exclusive=['week', 'month', 'luna', 'year', 'all'],
               help='Reports activity for the current day.')
 @click.option('-a', '--all', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('all'),
-              mutually_exclusive=['day', 'week', 'month', 'year'],
+              flag_value=_SHORTCUT_OPTIONS_VALUES['all'],
+              mutually_exclusive=['day', 'week', 'month', 'luna', 'year'],
               help='Reports all activities.')
 @click.option('-p', '--project', 'projects', multiple=True,
               help="Reports activity only for the given project. You can add "
@@ -354,13 +409,14 @@ _SHORTCUT_OPTIONS = ['all', 'year', 'month', 'week', 'day']
               help="Reports activity only for frames containing the given "
               "tag. You can add several tags by using this option multiple "
               "times")
-@click.option('-j', '--json', 'format_json', is_flag=True,
+@click.option('-j', '--json', 'format_json', cls=MutuallyExclusiveOption,
+              is_flag=True,
               help="Format the report in JSON instead of plain text")
 @click.option('-g/-G', '--pager/--no-pager', 'pager', default=None,
               help="(Don't) view output through a pager.")
 @click.pass_obj
-def report(watson, current, from_, to, projects,
-           tags, year, month, week, day, all, format_json, pager):
+def report(watson, current, from_, to, projects, tags, year, month,
+           week, day, luna, all, format_json, pager, aggregated=False):
     """
     Display a report of the time spent on each project.
 
@@ -375,6 +431,8 @@ def report(watson, current, from_, to, projects,
     `--day` sets the report timespan to the current day (beginning at 00:00h)
     and `--year`, `--month` and `--week` to the current year, month or week
     respectively.
+    The shortcut `--luna` sets the timespan to the current moon cycle with
+    the last full moon marking the start of the cycle.
 
     You can limit the report to a project or a tag using the `--project` and
     `--tag` options. They can be specified several times each to add multiple
@@ -453,16 +511,26 @@ def report(watson, current, from_, to, projects,
         }
     }
     """
+
+    # if the report is an aggregate report, add whitespace using this
+    # aggregate tab which will be prepended to the project name
+    if aggregated:
+        tab = '  '
+    else:
+        tab = ''
+
     try:
         report = watson.report(from_, to, current, projects, tags,
                                year=year, month=month, week=week, day=day,
-                               all=all)
+                               luna=luna, all=all)
     except watson.WatsonError as e:
         raise click.ClickException(e)
 
-    if format_json:
+    if format_json and not aggregated:
         click.echo(json.dumps(report, indent=4, sort_keys=True))
         return
+    elif format_json and aggregated:
+        return report
 
     lines = []
     # use the pager, or print directly to the terminal
@@ -473,7 +541,14 @@ def report(watson, current, from_, to, projects,
             lines.append(line)
 
         def _final_print(lines):
-            click.echo_via_pager('\n'.join(lines))
+            click.echo_via_pager(u'\n'.join(lines))
+    elif aggregated:
+
+        def _print(line):
+            lines.append(line)
+
+        def _final_print(lines):
+            pass
     else:
 
         def _print(line):
@@ -482,18 +557,32 @@ def report(watson, current, from_, to, projects,
         def _final_print(lines):
             pass
 
-    _print('{} -> {}\n'.format(
-        style('date', '{:ddd DD MMMM YYYY}'.format(
-            arrow.get(report['timespan']['from'])
-        )),
-        style('date', '{:ddd DD MMMM YYYY}'.format(
-            arrow.get(report['timespan']['to'])
+    # handle special title formatting for aggregate reports
+    if aggregated:
+        _print(u'{} - {}'.format(
+            style('date', '{:ddd DD MMMM YYYY}'.format(
+                arrow.get(report['timespan']['from'])
+            )),
+            style('time', '{}'.format(format_timedelta(
+                datetime.timedelta(seconds=report['time'])
+            )))
+        ))
+
+    else:
+        _print(u'{} -> {}\n'.format(
+            style('date', '{:ddd DD MMMM YYYY}'.format(
+                arrow.get(report['timespan']['from'])
+            )),
+            style('date', '{:ddd DD MMMM YYYY}'.format(
+                arrow.get(report['timespan']['to'])
+            ))
         ))
-    ))
 
     projects = report['projects']
+
     for project in projects:
-        _print('{project} - {time}'.format(
+        _print(u'{tab}{project} - {time}'.format(
+            tab=tab,
             time=style('time', format_timedelta(
                 datetime.timedelta(seconds=project['time'])
             )),
@@ -505,24 +594,142 @@ def report(watson, current, from_, to, projects,
             longest_tag = max(len(tag) for tag in tags or [''])
 
             for tag in tags:
-                _print('\t[{tag} {time}]'.format(
+                _print(u'\t[{tag} {time}]'.format(
                     time=style('time', '{:>11}'.format(format_timedelta(
                         datetime.timedelta(seconds=tag['time'])
                     ))),
-                    tag=style('tag', '{:<{}}'.format(
+                    tag=style('tag', u'{:<{}}'.format(
                         tag['name'], longest_tag
                     )),
                 ))
         _print("")
 
-    if len(projects) > 1:
+    # only show total time at the bottom for a project if it is not
+    # an aggregate report and there is greater than 1 project
+    if len(projects) > 1 and not aggregated:
         _print('Total: {}'.format(
             style('time', '{}'.format(format_timedelta(
                 datetime.timedelta(seconds=report['time'])
             )))
         ))
 
-    _final_print(lines)
+    # if this is a report invoked from `aggregate`
+    # return the lines
+    if aggregated:
+        return lines
+    else:
+        _final_print(lines)
+
+
+@cli.command()
+@click.option('-c/-C', '--current/--no-current', 'current', default=None,
+              help="(Don't) include currently running frame in report.")
+@click.option('-f', '--from', 'from_', cls=MutuallyExclusiveOption, type=Date,
+              default=arrow.now().replace(days=-7),
+              mutually_exclusive=_SHORTCUT_OPTIONS,
+              help="The date from when the report should start. Defaults "
+              "to seven days ago.")
+@click.option('-t', '--to', cls=MutuallyExclusiveOption, type=Date,
+              default=arrow.now(),
+              mutually_exclusive=_SHORTCUT_OPTIONS,
+              help="The date at which the report should stop (inclusive). "
+              "Defaults to tomorrow.")
+@click.option('-p', '--project', 'projects', multiple=True,
+              help="Reports activity only for the given project. You can add "
+              "other projects by using this option several times.")
+@click.option('-T', '--tag', 'tags', multiple=True,
+              help="Reports activity only for frames containing the given "
+              "tag. You can add several tags by using this option multiple "
+              "times")
+@click.option('-j', '--json', 'format_json', cls=MutuallyExclusiveOption,
+              is_flag=True,
+              help="Format the report in JSON instead of plain text")
+@click.option('-g/-G', '--pager/--no-pager', 'pager', default=None,
+              help="(Don't) view output through a pager.")
+@click.pass_obj
+@click.pass_context
+def aggregate(ctx, watson, current, from_, to, projects, tags,
+              format_json, pager):
+    """
+    Display a report of the time spent on each project aggregated by day.
+
+    If a project is given, the time spent on this project is printed.
+    Else, print the total for each root project.
+
+    By default, the time spent the last 7 days is printed. This timespan
+    can be controlled with the `--from` and `--to` arguments. The dates
+    must have the format `YEAR-MONTH-DAY`, like: `2014-05-19`.
+
+    You can limit the report to a project or a tag using the `--project` and
+    `--tag` options. They can be specified several times each to add multiple
+    projects or tags to the report.
+
+    If you are outputting to the terminal, you can selectively enable a pager
+    through the `--pager` option.
+
+    You can change the output format for the report from *plain text* to *JSON*
+    by using the `--json` option.
+
+    Example:
+
+    \b
+    $ watson aggregate
+    Wed 14 November 2018 - 5h 42m 22s
+      watson - 5h 42m 22s
+            [features     34m 06s]
+            [docs  5h 08m 16s]
+    \b
+    Thu 15 November 2018 - 00s
+    \b
+    Fri 16 November 2018 - 00s
+    \b
+    Sat 17 November 2018 - 00s
+    \b
+    Sun 18 November 2018 - 00s
+    \b
+    Mon 19 November 2018 - 5h 58m 52s
+      watson - 5h 58m 52s
+            [features  1h 12m 03s]
+            [docs  4h 46m 49s]
+    \b
+    Tue 20 November 2018 - 2h 50m 35s
+      watson - 2h 50m 35s
+            [features     15m 17s]
+            [docs  1h 37m 43s]
+            [website     57m 35s]
+    \b
+    Wed 21 November 2018 - 01m 17s
+      watson - 01m 17s
+            [docs     01m 17s]
+    """
+    delta = (to - from_).days
+    lines = []
+
+    for i in range(delta + 1):
+        offset = datetime.timedelta(days=i)
+        from_offset = from_ + offset
+        output = ctx.invoke(report, current=current, from_=from_offset,
+                            to=from_offset, projects=projects, tags=tags,
+                            format_json=format_json, pager=pager,
+                            aggregated=True)
+
+        if format_json:
+            lines.append(output)
+        else:
+            # if there is no activity for the day, append a newline
+            # this ensures even spacing throughout the report
+            if (len(output)) == 1:
+                output[0] += '\n'
+
+            lines.append(u'\n'.join(output))
+
+    if format_json:
+        click.echo(json.dumps(lines, indent=4, sort_keys=True))
+    elif pager or (pager is None and
+                   watson.config.getboolean('options', 'pager', True)):
+        click.echo_via_pager(u'\n\n'.join(lines))
+    else:
+        click.echo(u'\n\n'.join(lines))
 
 
 @cli.command()
@@ -536,23 +743,27 @@ def report(watson, current, from_, to, projects,
               help="The date at which the log should stop (inclusive). "
               "Defaults to tomorrow.")
 @click.option('-y', '--year', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('year'),
+              flag_value=_SHORTCUT_OPTIONS_VALUES['year'],
               mutually_exclusive=['day', 'week', 'month', 'all'],
               help='Reports activity for the current year.')
 @click.option('-m', '--month', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('month'),
+              flag_value=_SHORTCUT_OPTIONS_VALUES['month'],
               mutually_exclusive=['day', 'week', 'year', 'all'],
               help='Reports activity for the current month.')
+@click.option('-l', '--luna', cls=MutuallyExclusiveOption, type=Date,
+              flag_value=_SHORTCUT_OPTIONS_VALUES['luna'],
+              mutually_exclusive=['day', 'week', 'month', 'year', 'all'],
+              help='Reports activity for the current moon cycle.')
 @click.option('-w', '--week', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('week'),
+              flag_value=_SHORTCUT_OPTIONS_VALUES['week'],
               mutually_exclusive=['day', 'month', 'year', 'all'],
               help='Reports activity for the current week.')
 @click.option('-d', '--day', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('day'),
+              flag_value=_SHORTCUT_OPTIONS_VALUES['day'],
               mutually_exclusive=['week', 'month', 'year', 'all'],
               help='Reports activity for the current day.')
 @click.option('-a', '--all', cls=MutuallyExclusiveOption, type=Date,
-              flag_value=get_start_time_for_period('all'),
+              flag_value=_SHORTCUT_OPTIONS_VALUES['all'],
               mutually_exclusive=['day', 'week', 'month', 'year'],
               help='Reports all activities.')
 @click.option('-p', '--project', 'projects', multiple=True,
@@ -568,7 +779,7 @@ def report(watson, current, from_, to, projects,
               help="(Don't) view output through a pager.")
 @click.pass_obj
 def log(watson, current, from_, to, projects, tags, year, month, week, day,
-        all, format_json, pager):
+        luna, all, format_json, pager):
     """
     Display each recorded session during the given timespan.
 
@@ -580,6 +791,8 @@ def log(watson, current, from_, to, projects, tags, year, month, week, day,
     `--day` sets the log timespan to the current day (beginning at 00:00h)
     and `--year`, `--month` and `--week` to the current year, month or week
     respectively.
+    The shortcut `--luna` sets the timespan to the current moon cycle with
+    the last full moon marking the start of the cycle.
 
     If you are outputting to the terminal, you can selectively enable a pager
     through the `--pager` option.
@@ -617,7 +830,7 @@ def log(watson, current, from_, to, projects, tags, year, month, week, day,
             02cb269  09:53 to 12:43   2h 50m 07s  apollo11  [wheels]
             1070ddb  13:48 to 16:17   2h 29m 11s  voyager1  [antenna, sensors]
     """  # noqa
-    for start_time in (_ for _ in [day, week, month, year, all]
+    for start_time in (_ for _ in [day, week, month, luna, year, all]
                        if _ is not None):
         from_ = start_time
 
@@ -693,10 +906,11 @@ def log(watson, current, from_, to, projects, tags, year, month, week, day,
         )
 
         _print("\n".join(
-            "\t{id}  {start} to {stop}  {delta:>11}  {project}{tags}".format(
+            u"\t{id}  {start} to {stop}  {delta:>11}  {project}{tags}".format(
                 delta=format_timedelta(frame.stop - frame.start),
-                project=style('project',
-                              '{:>{}}'.format(frame.project, longest_project)),
+                project=style('project', u'{:>{}}'.format(
+                    frame.project, longest_project
+                )),
                 pad=longest_project,
                 tags=(" "*2 if frame.tags else "") + style('tags', frame.tags),
                 start=style('time', '{:HH:mm}'.format(frame.start)),
@@ -774,6 +988,45 @@ def frames(watson):
         click.echo(style('short_id', frame.id))
 
 
+@cli.command(context_settings={'ignore_unknown_options': True})
+@click.argument('args', nargs=-1)
+@click.option('-f', '--from', 'from_', required=True, type=Date,
+              help="Date and time of start of tracked activity")
+@click.option('-t', '--to', required=True, type=Date,
+              help="Date and time of end of tracked activity")
+@click.pass_obj
+def add(watson, args, from_, to):
+    """
+    Add time for project with tag(s) that was not tracked live.
+
+    Example:
+
+    \b
+    $ watson add --from "2018-03-20 12:00:00" --to "2018-03-20 13:00:00" \\
+     programming +addfeature
+    """
+    # parse project name from args
+    project = ' '.join(
+        itertools.takewhile(lambda s: not s.startswith('+'), args)
+    )
+
+    # Parse all the tags
+    tags = parse_tags(args)
+
+    # add a new frame, call watson save to update state files
+    frame = watson.add(project=project, tags=tags, from_date=from_, to_date=to)
+    click.echo(
+        u"Adding project {}{}, started {} and stopped {}. (id: {})".format(
+            style('project', frame.project),
+            (" " if frame.tags else "") + style('tags', frame.tags),
+            style('time', frame.start.humanize()),
+            style('time', frame.stop.humanize()),
+            style('short_id', frame.id)
+        )
+    )
+    watson.save()
+
+
 @cli.command(context_settings={'ignore_unknown_options': True})
 @click.argument('id', required=False)
 @click.pass_obj
@@ -821,27 +1074,51 @@ def edit(watson, id):
         data['stop'] = frame.stop.format(datetime_format)
 
     text = json.dumps(data, indent=4, sort_keys=True, ensure_ascii=False)
-    output = click.edit(text, extension='.json')
 
-    if not output:
-        click.echo("No change made.")
-        return
+    start = None
+    stop = None
 
-    try:
-        data = json.loads(output)
-        project = data['project']
-        tags = data['tags']
-        start = arrow.get(data['start'], datetime_format).replace(
-            tzinfo=local_tz).to('utc')
-        stop = arrow.get(data['stop'], datetime_format).replace(
-            tzinfo=local_tz).to('utc') if id else None
-    except (ValueError, RuntimeError) as e:
-        raise click.ClickException("Error saving edited frame: {}".format(e))
-    except KeyError:
-        raise click.ClickException(
-            "The edited frame must contain the project, start and stop keys."
-        )
+    # enter into while loop until succesful and validated
+    #  edit has been performed
+    while True:
+        output = click.edit(text, extension='.json')
 
+        if not output:
+            click.echo("No change made.")
+            return
+
+        try:
+            data = json.loads(output)
+            project = data['project']
+            tags = data['tags']
+            start = arrow.get(data['start'], datetime_format).replace(
+                tzinfo=local_tz).to('utc')
+            stop = arrow.get(data['stop'], datetime_format).replace(
+                tzinfo=local_tz).to('utc') if id else None
+            # if start time of the project is not before end time
+            #  raise ValueException
+            if not watson.is_started and start > stop:
+                raise ValueError(
+                    "Task cannot end before it starts.")
+            # break out of while loop and continue execution of
+            #  the edit function normally
+            break
+        except (ValueError, RuntimeError) as e:
+            click.echo(u"Error while parsing inputted values: {}".format(e),
+                       err=True)
+        except KeyError:
+            click.echo(
+                "The edited frame must contain the project, \
+                start, and stop keys.", err=True)
+        # we reach here if exception was thrown, wait for user
+        #  to acknowledge the error before looping in while and
+        #  showing user the editor again
+        click.pause(err=True)
+        # use previous entered values to the user in editor
+        #  instead of original ones
+        text = output
+
+    # we reach this when we break out of the while loop above
     if id:
         watson.frames[id] = (project, start, stop, tags)
     else:
@@ -849,8 +1126,8 @@ def edit(watson, id):
 
     watson.save()
     click.echo(
-        "Edited frame for project {project}{tags}, from {start} to {stop} "
-        "({delta})".format(
+        u"Edited frame for project {project}{tags}, from {start} to {stop} "
+        u"({delta})".format(
             delta=format_timedelta(stop - start) if stop else '-',
             project=style('project', project),
             tags=(" " if tags else "") + style('tags', tags),
@@ -881,8 +1158,8 @@ def remove(watson, id, force):
 
     if not force:
         click.confirm(
-            "You are about to remove frame "
-            "{project}{tags} from {start} to {stop}, continue?".format(
+            u"You are about to remove frame "
+            u"{project}{tags} from {start} to {stop}, continue?".format(
                 project=style('project', frame.project),
                 tags=(" " if frame.tags else "") + style('tags', frame.tags),
                 start=style('time', '{:HH:mm}'.format(frame.start)),
@@ -956,11 +1233,11 @@ def config(context, key, value, edit):
 
     if value is None:
         if not wconfig.has_section(section):
-            raise click.ClickException("No such section {}".format(section))
+            raise click.ClickException(u"No such section {}".format(section))
 
         if not wconfig.has_option(section, option):
             raise click.ClickException(
-                "No such option {} in {}".format(option, section)
+                u"No such option {} in {}".format(option, section)
             )
 
         click.echo(wconfig.get(section, option))
@@ -1103,7 +1380,7 @@ def merge(watson, frames_with_conflict, force):
             'tags': original_frame.tags
         }
         click.echo("frame {}:".format(style('short_id', original_frame.id)))
-        click.echo("{}".format('\n'.join('<' + line for line in json.dumps(
+        click.echo(u"{}".format('\n'.join('<' + line for line in json.dumps(
             original_frame_data, indent=4, ensure_ascii=False).splitlines())))
         click.echo("---")
 
@@ -1179,7 +1456,7 @@ def rename(watson, rename_type, old_name, new_name):
         except ValueError as e:
             raise click.ClickException(style('error', str(e)))
         else:
-            click.echo('Renamed tag "{}" to "{}"'.format(
+            click.echo(u'Renamed tag "{}" to "{}"'.format(
                             style('tag', old_name),
                             style('tag', new_name)
                        ))
@@ -1189,13 +1466,13 @@ def rename(watson, rename_type, old_name, new_name):
         except ValueError as e:
             raise click.ClickException(style('error', str(e)))
         else:
-            click.echo('Renamed project "{}" to "{}"'.format(
+            click.echo(u'Renamed project "{}" to "{}"'.format(
                             style('project', old_name),
                             style('project', new_name)
                        ))
     else:
         raise click.ClickException(style(
             'error',
-            'You have to call rename with type "project" or "tag"; '
-            'you supplied "%s"' % rename_type
+            u'You have to call rename with type "project" or "tag"; '
+            u'you supplied "%s"' % rename_type
         ))
